diff --git a/lisp/progmodes/gdb-mi.el b/lisp/progmodes/gdb-mi.el
index bc0070d2630..f7e96a2f19e 100644
--- a/lisp/progmodes/gdb-mi.el
+++ b/lisp/progmodes/gdb-mi.el
@@ -1960,19 +1960,28 @@ breakpoint-disabled
   :group 'gdb)
 
 
+(rx-define gdb-python-guile-commands
+  (or "python" "python-interactive" "pi" "guile" "guile-repl" "gr"))
+
 (defvar gdb-python-guile-commands-regexp
-  "python\\|python-interactive\\|pi\\|guile\\|guile-repl\\|gr"
+  (rx gdb-python-guile-commands)
   "Regexp that matches Python and Guile commands supported by GDB.")
 
 (defvar gdb-control-commands-regexp
-  (concat
-   "^\\("
-   "comm\\(a\\(n\\(ds?\\)?\\)?\\)?\\|if\\|while"
-   "\\|def\\(i\\(ne?\\)?\\)?\\|doc\\(u\\(m\\(e\\(nt?\\)?\\)?\\)?\\)?\\|"
-   gdb-python-guile-commands-regexp
-   "\\|while-stepping\\|stepp\\(i\\(ng?\\)?\\)?\\|ws\\|actions"
-   "\\|expl\\(o\\(re?\\)?\\)?"
-   "\\)\\([[:blank:]]+\\([^[:blank:]]*\\)\\)*$")
+  (rx bol
+      (or "comm" "comma" "comman" "command" "commands"
+          "if" "while"
+          "def" "defi" "defin" "define"
+          "doc" "docu" "docum" "docume" "documen" "document"
+          "while-stepping"
+          "stepp" "steppi" "steppin" "stepping"
+          "ws" "actions"
+          "expl" "explo" "explor" "explore"
+          gdb-python-guile-commands)
+      (? (+ blank)
+         (group       ; Group 1 contains the command arguments.
+          (* nonl)))
+      eol)
   "Regexp matching GDB commands that enter a recursive reading loop.
 As long as GDB is in the recursive reading loop, it does not expect
 commands to be prefixed by \"-interpreter-exec console\".")
@@ -2033,7 +2042,7 @@ gdb-send
   ;; Python and Guile commands that have an argument don't enter the
   ;; recursive reading loop.
   (let* ((control-command-p (string-match gdb-control-commands-regexp string))
-         (command-arg (and control-command-p (match-string 3 string)))
+         (command-arg (and control-command-p (match-string 1 string)))
          (python-or-guile-p (string-match gdb-python-guile-commands-regexp
                                           string)))
     (if (and control-command-p
